require 'json'
require 'to_slug'
require 'awesome_print'
require 'ezpaas/server/routes/api'
require 'ezpaas/helpers/container_manager'
require 'ezpaas/models/app'

module EzPaaS
  module Server
    module Routes
      class Apps < API

        configure_api

        desc 'Returns all apps'
        get do
          { apps: Models::App.all.to_a.map { |e| e.to_hash } }
        end

        desc 'Create an app'
        params do
          requires :name, type: String, desc: "The new app's name"
        end
        post do
          name = params[:name]
          slug = name.to_slug
          error!("App named #{slug} already exists", 409) unless Models::App.where(name: slug).empty?
          a = Models::App.new
          a.name = slug
          a.save
          {
            app: a.to_hash
          }
        end

        desc 'Delete an app'
        params do
          requires :name, type: String, desc: "The app's name"
        end
        delete do
          name = params[:name]
          slug = name.to_slug
          error!("Could not find app named #{slug}", 404) unless app = Models::App.where(name: slug).first

          scale = app.scale || {}
          container_count = scale.values.reduce(0) { |x, y| x + y }

          error!("App #{slug} currently has #{container_count} containers running. Please scale it to 0 before trying to delete it.", 409) unless container_count == 0

          # ensure images are cleared out
          manager = Helpers::ContainerManager.new
          manager.undeploy_app(slug)

          app.destroy
        end

      end
    end
  end
end
