require 'sinatra'
require 'pathname'
require 'uri'
require 'awesome_print'
require 'excon'

require 'ezpaas/helpers/container_manager'

module EzPaaS
  module Server
    module Routes
      class Proxy < Sinatra::Application

        %i(get post delete patch put head options).each do |method|
          send method, '/:app/*' do
            full_path = request.path_info

            components = Pathname.new(full_path).each_filename.to_a
            app = components.shift

            path = Pathname.new('/').join(components.join('/')).to_s

            unless request.query_string.empty?
              path += '?' + request.query_string
            end

            (container, port) = get_container(app)

            unless container and port
              return [404, 'container not found']
            end

            destination = "http://127.0.0.1:#{port}"

            connection = Excon.new(destination)

            response = connection.request(method: method, path: path)

            headers = response.headers
            headers['X-Container'] = container

            [response.status, headers, response.body]
          end
        end

        private

        def headers_hash
          # https://stackoverflow.com/questions/6317705/rackrequest-how-do-i-get-all-headers
          Hash[*env.select {|k,v| k.start_with? 'HTTP_'}
               .collect {|k,v| [k.sub(/^HTTP_/, ''), v]}
               .collect {|k,v| [k.split('_').collect(&:capitalize).join('-'), v]}
               .sort
               .flatten]
        end

        def get_container(app)
          manager = Helpers::ContainerManager.new
          options = manager.http_destinations(app)
          return nil if options.empty?
          key = options.keys.sample
          [key, options[key]]
        end

      end

    end
  end
end
